@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Tester
 #
-# This software is Copyright (c) 2013-2016 by Dominique Dumont.
+# This software is Copyright (c) 2013-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -9,7 +9,7 @@
 #
 package Config::Model::Tester;
 # ABSTRACT: Test framework for Config::Model
-$Config::Model::Tester::VERSION = '2.058';
+$Config::Model::Tester::VERSION = '2.059';
 use warnings;
 use strict;
 use locale;
@@ -46,7 +46,7 @@ require Exporter;
 $File::Copy::Recursive::DirPerms = 0755;
 
 sub setup_test {
-    my ( $app_to_test, $t_name, $wr_root, $trace, $setup ) = @_;
+    my ( $app_to_test, $t_name, $wr_root, $trace, $t_data ) = @_;
 
     # cleanup before tests
     $wr_root->remove_tree();
@@ -61,11 +61,11 @@ sub setup_test {
         if $conf_dir and $conf_file_name;
 
     my $ex_dir = path('t')->child('model_tests.d', "$app_to_test-examples");
-    my $ex_data = $ex_dir->child($t_name);
+    my $ex_data = $ex_dir->child($t_data->{data_from} // $t_name);
 
     my @file_list;
 
-    if ($setup) {
+    if (my $setup = $t_data->{setup}) {
         foreach my $file (keys %$setup) {
             my $map = $setup->{$file} ;
             my $destination_str
@@ -176,10 +176,10 @@ sub run_update {
 }
 
 sub load_instructions {
-    my ($root,$t,$trace) = @_ ;
+    my ($root,$steps,$trace) = @_ ;
 
-    print "Loading $t->{load}\n" if $trace ;
-    $root->load( $t->{load} );
+    print "Loading $steps\n" if $trace ;
+    $root->load( $steps );
     ok( 1, "load called" );
 }
 
@@ -441,14 +441,19 @@ sub run_model_test {
         note("Beginning subtest $app_to_test $t_name");
 
         my ($wr_dir, $wr_dir2, $conf_file, $ex_data, @file_list)
-            = setup_test ($app_to_test, $t_name, $wr_root,$trace, $t->{setup});
+            = setup_test ($app_to_test, $t_name, $wr_root,$trace, $t);
 
         write_config_file($conf_dir,$wr_dir,$t);
 
+        my $inst_name = "$app_to_test-" . $t_name;
+
+        die "Duplicated test name $t_name for app $app_to_test\n"
+            if $model->has_instance ($inst_name);
+
         my $inst = $model->instance(
             root_class_name => $model_to_test,
             root_dir        => $wr_dir->stringify,
-            instance_name   => "$app_to_test-" . $t_name,
+            instance_name   => $inst_name,
             application     => $app_to_test,
             config_file     => $t->{config_file} ,
             check           => $t->{load_check} || 'yes',
@@ -461,7 +466,7 @@ sub run_model_test {
 
         run_update($inst,$wr_dir,$t) if $t->{update};
 
-        load_instructions ($root,$t,$trace) if $t->{load} ;
+        load_instructions ($root,$t->{load},$trace) if $t->{load} ;
 
         dump_tree ('before fix '.$app_to_test , $root, 'full', $t->{no_warnings}, $t->{check_before_fix}, $trace)
             if $t->{check_before_fix};
@@ -487,6 +492,8 @@ sub run_model_test {
 
         my $i2_root = create_second_instance ($app_to_test, $t_name, $wr_dir, $wr_dir2,$t, $config_dir_override);
 
+        load_instructions ($i2_root,$t->{load2},$trace) if $t->{load2} ;
+
         my $p2_dump = dump_tree("second $app_to_test", $i2_root, 'custom', $t->{no_warnings},{}) ;
 
         unified_diff;
@@ -515,6 +522,17 @@ sub translate_test_data {
     map {$t->{full_dump}{$_} = delete $t->{$_} if $t->{$_}; } qw/dump_warnings dump_errors/;
 }
 
+sub create_model_object {
+    my $new_model ;
+    eval { $new_model = Config::Model->new(); } ;
+    if ($@) {
+        # necessary to run smoke test (no Config::Model to avoid dependency loop)
+        plan skip_all => 'Config::Model is not loaded' ;
+        return;
+    }
+    return $new_model;
+}
+
 sub run_tests {
     my ( $arg, $test_only_app, $do ) = @_;
 
@@ -532,12 +550,6 @@ sub run_tests {
         Log::Log4perl->easy_init( $log ? $WARN : $ERROR );
     }
 
-    eval { $model = Config::Model->new(); } ;
-    if ($@) {
-        plan skip_all => 'Config::Model is not loaded' ;
-        return;
-    }
-
     Config::Model::Exception::Any->Trace(1) if $arg =~ /e/;
 
     ok( 1, "compiled" );
@@ -550,6 +562,8 @@ sub run_tests {
     foreach my $app_to_test_conf (@group_of_tests) {
         my ($app_to_test) = ( $app_to_test_conf =~ m!\.d/([\w\-]+)-test-conf! );
         next if ( $test_only_app and $test_only_app ne $app_to_test ) ;
+        $model = create_model_object();
+        return unless $model;
         run_model_test($app_to_test, $app_to_test_conf, $do, $model, $trace, $wr_root) ;
     }
 
@@ -572,7 +586,7 @@ Config::Model::Tester - Test framework for Config::Model
 
 =head1 VERSION
 
-version 2.058
+version 2.059
 
 =head1 SYNOPSIS
 
@@ -764,8 +778,9 @@ L<test for shellvar backend|https://github.com/dod38fr/config-model/blob/master/
 
 =head2 Test specification with arbitrary file names
 
-In some models (e.g. C<Multistrap>, the config file is chosen by the user.
-In this case, the file name must be specified for each tests case:
+In some models like C<Multistrap>, the config file is chosen by the
+user. In this case, the file name must be specified for each tests
+case:
 
  $model_to_test = "Multistrap";
 
@@ -777,7 +792,29 @@ In this case, the file name must be specified for each tests case:
     },
  );
 
-See actual L<multistrap test|https://github.com/dod38fr/config-model/blob/master/t/model_tests.d/multistrap-test-conf.pl>.
+See the actual L<multistrap test|https://github.com/dod38fr/config-model/blob/master/t/model_tests.d/multistrap-test-conf.pl>.
+
+=head2 Re-use test data
+
+When the input data for test is quite complex (several files), it may
+be interested to re-use these data for other tests case. Knowing that
+test name must must unique, you can re-use test data with C<data_from>
+parameter. For instance:
+
+  @tests = (
+    {
+        name  => 'some-test',
+        # ...
+    },
+    {
+        name  => 'some-other-test',
+        data_from  => 'some-test',    # re-use data from test above
+        # ...
+    },
+
+See
+L<plainfile backend test|https://github.com/dod38fr/config-model/blob/master/t/model_tests.d/backend-plainfile-test-conf.pl>
+for a real life example.
 
 =head2 Test scenario
 
@@ -973,6 +1010,9 @@ specify a subref to alter the file list:
         push @$list_ref, "/debian/source/format" ;
     },
 
+Note that actual and expected file lists are sorted before check,
+adding a file can be done with C<push>.
+
 =item *
 
 Copy all config data from C<< wr_root/<subtest name>/ >>
@@ -982,7 +1022,7 @@ the original configuration.
 
 =item *
 
-Create another configuration instance to read the conf file that was just copied
+Create a second configuration instance to read the conf file that was just copied
 (configuration data is checked.)
 
 =item *
@@ -992,6 +1032,16 @@ some errors were ignored and cannot be fixed) with C<< load_check2 => 'no' >>
 
 =item *
 
+Optionally load configuration data in the second instance. You should
+design this config data to suppress any error or warning that occur in
+the step below. E.g:
+
+    load2 => 'binary:seaview',
+
+See L<Config::Model::Loader> for the syntax of the string accepted by C<load2> parameter.
+
+=item *
+
 Compare data read from original data.
 
 =item *
@@ -1104,7 +1154,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2013-2016 by Dominique Dumont.
+This software is Copyright (c) 2013-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
